###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__author__ = '$Author: morten $'
__version__ = '$Revision: 1.8 $'


import popen2
import thread
import string
import tempfile
import time

class fetchmail:

	success_codes = {
		0:'One or more messages were successfully retrieved',
		1:'No new messages, no messages retrieved'
		}

	fatal_error_codes = {
		2:'Socket error',
		3:'User authentication error',
		4:'Fatal protocol error detected',
		5:'Syntax error in configuration file',
		6:'Run control file had bad permissions', # This shouldn't happen
		7:'Fetchmail timed out, or server error',
		8:'Client side exlusion error, another fetchmail may be running',
		9:'Server responded lock busy, try again later',
		10:'Fetchmail failed on SMTP port opening, or transaction',
		11:'Fatal DNS error, error encountered upon lookup',
		12:'BSTP batch file could not be opened',
		13:'Poll terminated by fetch limit',
		23:'Internal error, see standard error stream for info'
		}


	def __init__(self, server, username, local_username,
				 options, protocol='auto', uidl=''):
		"""An interface to fetchmail.

		This class lets us retrieve emails from various accounts and
		send them to a local user.
		"""

		self.server = server
		self.username = username
		self.local_username = local_username
		self.options = options
		self.protocol = protocol
		self.uidl = uidl


	def fetch(self, password):

		thread.start_new_thread(self._fetch,(password,))


	def _fetch(self, password):

		fetchmail_command = 'fetchmail -f -'

		if self.protocol == 'pop3' and self.uidl:

			uid_file = open(tempfile.mktemp(),'w')
			uid_file.write(self.uidl)
			uid_file.close()
			
			fetchmail_command = fetchmail_command + (' -i %s' % uid_file.name)

		self.fetchmail_pipe = popen2.Popen3(fetchmail_command, capturestderr=1)

		stdin = self.fetchmail_pipe.tochild

		stdin.write("""set properties ""\n""")
		stdin.write("""poll  %s with proto %s\n""" % \
					(self.server,self.protocol))
		stdin.write(
			"""user %s there with password %s is %s here\n""" % \
			(self.username,password,self.local_username))

		if self.options:

			stdin.write("\noptions %s" % (string.join(self.options,' ')))
                    
		stdin.flush()
		stdin.close()


		poll_result = self.fetchmail_pipe.wait()
		print poll_result, self.fetchmail_pipe.poll()

		print 'fetchmail finished', poll_result

		# The fetchmail process has completed

		if poll_result in self.success_codes.keys():

			self.status_message = self.success_codes[poll_result]

			# 0: success (retrieved messages)
			# 1: success (no retrieved messages, (no new))
			# 256: success (no retrieved messages)

			print poll_result

		else:

			if self.uidl:

				uid_file = open(uid_file.name,'r')
				uid_file.seek(len(self.uid))
				self.uid = self.uid + uid_file.read()

				uid_file.close()
				os.unlink(uid_file.name)



if __name__ == '__main__':

	fi = fetchmail('esol.no','test123','morten',['flush'])
